#!/usr/bin/env bash


## --- Variables (default Values)
## ---
#_zone_file_dir=/etc/bind/master
_zone_file_dir=/root/tmp/master

_serial_new=`date +%Y%m%d01`

_zone_file_suffix=zone
## ---
## --- End: Variables (default Values)


## --- some functions
## ---
echononl(){
   echo X\\c > /tmp/shprompt$$
   if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
      echo -e -n "$*\\c" 1>&2
   else
       echo -e -n "$*" 1>&2
   fi
   rm /tmp/shprompt$$
}

fatal(){
   echo ""
   echo -e "[ \033[31m\033[1mError\033[m ]: $*"
   echo ""
   echo -e "\t\033[31m\033[1mScript is canceled\033[m\033[m"
   echo ""
   exit 1
}

warn (){
   echo ""
   echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
   echo ""
}

info (){
   echo ""
   echo -e "\t[ \033[33m\033[1mInfo\033[m ]: $*"
   echo ""
}

ok (){
   echo ""
   echo -e "\t[ \033[36m\033[1mOk\033[m ]: $*"
   echo ""
}

error(){
   echo ""
   echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
   echo ""
}

echo_ok() {
   echo -e "\033[75G[ \033[32mok\033[m ]"
}
echo_failed(){
   echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
}
echo_skipped() {
   echo -e "\033[75G[ \033[33m\033[1mskipped\033[m ]"
}

containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}


## - Test of valid IPv4 Address
## -
## -    Returns 0 if valid, > 0 otherwise
## -
is_valid_ipv4() {
  local -a octets=( ${1//\./ } )
  local RETURNVALUE=0

  # return an error if the IP doesn't have exactly 4 octets
  [[ ${#octets[@]} -ne 4 ]] && return 1

  for octet in ${octets[@]}
  do
    if [[ ${octet} =~ ^[0-9]{1,3}$ ]]
    then # shift number by 8 bits, anything larger than 255 will be > 0
      ((RETURNVALUE += octet>>8 ))
    else # octet wasn't numeric, return error
      return 1
    fi
  done
  return ${RETURNVALUE}
}

is_valid_ipv6() {
   local _ipv6=$1

   if [ "$1" != "${1#[0-9a-f]*:}" ] \
      && [ "$1" = "${1#*[^0-9a-f:]}" ] \
      && [ "${1#*[0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f]}" = "${1#*:*:*:*:*:*:*:*:*:}" ]; then
      return 0
   else
      return 1
   fi
}

## ---
## --- END: functions

echo ""
echo -e "\033[32m--\033[m"
echo ""
echo "Insert IPv4-Address(es) for which TTL should be set to zone-file default."
echo ""
echo "For multiple IPv4 Addresses, insert a blank separated list"
echo ""
echo -e "Type \"\033[33mNone\033[m\" if no IPv4 address should be changed."
echo ""
IPv4_ADDRESS=
_set_ipv4=true
while [ "X$IPv4_ADDRESS" = "X" ]; do
   echononl "IPv4-Address: "
   read IPv4_ADDRESS
   ## - To lower case
   IPv4_ADDRESS=${IPv4_ADDRESS,,}
   if [ "X$IPv4_ADDRESS" = "X" ]; then
      echo -e "\n\t\033[33m\033[1mAn entry is required!\033[m\n"
      IPv4_ADDRESS=""
      continue
   fi
   if [ "$IPv4_ADDRESS" = "none" ];then
      _set_ipv4=false
      break
   fi
   if ! is_valid_ipv4 $IPv4_ADDRESS ; then
      echo -e "\n\t\033[33m\033[1m$IPv4_ADDRESS\033[m is NOT a valid IPv4 Address\n"
      IPv4_ADDRESS=""
      continue
   fi
done


echo ""
echo -e "\033[32m--\033[m"
echo ""
echo "Insert IPv6-Address(es) for which TTL should be set to zone-file default."
echo ""
echo "For multiple IPv6 Addresses, insert a blank separated list"
echo ""
echo -e "Type \"\033[33mNone\033[m\" if no IPv6 address should be changed."
echo ""
IPv6_ADDRESS=
_set_ipv6=true
while [ "X$IPv6_ADDRESS" = "X" ]; do
   echononl "IPv6-Address: "
   read IPv6_ADDRESS
   ## - To lower case
   IPv6_ADDRESS=${IPv6_ADDRESS,,}
   if [ "X$IPv6_ADDRESS" = "X" ]; then
      echo -e "\n\t\033[33m\033[1mAn entry is required!\033[m\n"
      IPv6_ADDRESS=""
      continue
   fi
   if [ "$IPv6_ADDRESS" = "none" ];then
      _set_ipv6=false
      break
   fi
   if ! is_valid_ipv6 $IPv6_ADDRESS ; then
      echo -e "\n\t\033[33m\033[1m$IPv6_ADDRESS\033[m is NOT a valid IPv6 Address\n"
      IPv6_ADDRESS=""
      continue
   fi
done


echo ""
echo -e "\033[32m--\033[m"
echo ""
echo "Insert the directory, where your zone-files resides."
echo ""
echo ""
ZONE_FILE_DIR=
while [ "X$ZONE_FILE_DIR" = "X" ]; do
   echononl "Zone File Directory [$_zone_file_dir]: "
   read ZONE_FILE_DIR
   if [ "X$ZONE_FILE_DIR" = "X" ]; then
      ZONE_FILE_DIR=$_zone_file_dir
   fi
   if [ ! -d $ZONE_FILE_DIR ]; then
      echo -e "\n\tDirectory \033[33m\033[1m$ZONE_FILE_DIR\033[m does NOT exist!\n"
      ZONE_FILE_DIR=
   fi
done

echo ""
echo -e "\033[32m--\033[m"
echo ""
echo "Insert the file-suffix of thr zone-files"
echo ""
echo ""
ZONE_FILE_SUFFIX=
echononl "Suffix of Zone Files [$_zone_file_suffix]: "
read ZONE_FILE_SUFFIX
if [ "X$ZONE_FILE_SUFFIX" = "X" ]; then
   ZONE_FILE_SUFFIX=$_zone_file_suffix
fi


if ! $_set_ipv6 && ! $_set_ipv4 ; then
   fatal "No IP-Adresses given for changing their TTL to the zone-file default.."
fi

clear
echo ""
echo ""
echo -e "\033[21G\033[32mChange TTL settings to the zonefile default value\033[m"
echo ""

echo ""
if $_set_ipv4 ; then
   echo "IPv4 Address(es)..................: $IPv4_ADDRESS"
else
   echo -e "IPv4 Address(es)..................: \033[33mNone\033[m"
fi
if $_set_ipv6 ; then
   echo "IPv6 Address(es)..................: $IPv6_ADDRESS"
else
   echo -e "IPv6 Address(es)..................: \033[33mNone\033[m"
fi
echo ""
echo "New TTL...........................: Zonefile default"
echo ""
echo "Zone File Directory...............: $ZONE_FILE_DIR"
echo "Zone File Suffix..................: $ZONE_FILE_SUFFIX"

echo ""
OK=
while [ "$OK" != "yes" -o "$OK" != "no" ] ; do
   echononl "Parameters ok? [yes/no]: "
   read OK
   ## - To lower case
   OK=${OK,,}
   if [ "X$OK" = "X" ]; then
      echo -e "\n\t\033[33m\033[1mAn entry is required!\033[m\n"
      OK=""
      continue
   fi
   if [ "$OK" = "yes" -o "$OK" = "no" ] ; then
      break
   else
      OK=""
      continue
   fi
   echo -e "\n\t\033[33m\033[1mWrong entry!\033[m\n"
done
[[ $OK = "yes" ]] || fatal Repeat execution with different parameters


cp -a $ZONE_FILE_DIR ${ZONE_FILE_DIR}.BAK-`date +%Y-%m-%d-%H%M`

ipv4_addresses_arr=()
for _ipv4_address in $IPv4_ADDRESS ; do
   containsElement "$_ipv4_address"  "${ipv4_addresses_arr[@]}" && continue
   ipv4_addresses_arr+=("$_ipv4_address")
done

ipv6_addresses_arr=()
for _ipv6_address in $IPv6_ADDRESS ; do
   containsElement "$_ipv6_address"  "${ipv6_addresses_arr[@]}" && continue
   ipv6_addresses_arr+=("$_ipv6_address")
done

zonefiles_arr=()
if $_set_ipv4 ; then
   for _ipv4_address in "${ipv4_addresses_arr[@]}" ; do
      _zone_files_ipv4=`grep -l -e "$_ipv4_address" ${ZONE_FILE_DIR}/*.$ZONE_FILE_SUFFIX`
      if [ ${#zonefiles_arr[@]} -eq 0 ] ; then
         for _zone_file in "$_zone_files_ipv4" ; do
            zonefiles_arr+=("$_zone_file")
         done
      else
         for _zone_file in "$_zone_files_ipv4" ; do
            containsElement "$_zone_file"  "${zonefiles_arr[@]}" && continue
            zonefiles_arr+=("$_zone_file")
         done
      fi
   done
fi

if $_set_ipv6 ; then
   for _ipv6_address in "${ipv6_addresses_arr[@]}" ; do
      _zone_files_ipv6=`grep -l -e "$_ipv6_address" ${ZONE_FILE_DIR}/*.$ZONE_FILE_SUFFIX`
      if [ ${#zonefiles_arr[@]} -eq 0 ] ; then
         for _zone_file in "$_zone_files_ipv6" ; do
            zonefiles_arr+=("$_zone_file")
         done
      else
         for _zone_file in "$_zone_files_ipv6" ; do
            containsElement "$_zone_file"  "${zonefiles_arr[@]}" && continue
            zonefiles_arr+=("$_zone_file")
         done
      fi
   done
fi


for zone_file in ${zonefiles_arr[@]} ; do

   echo -e "\n\tconverting $zone_file .."
      
   _replaced=false

   ## - calculate new serial
   ## -
   declare -i __serial=`grep -e "[0-9]\{10\}" $zone_file | grep serial | awk '{print$1}'`
   while [ ! $_serial_new -gt $__serial ]; do
      let _serial_new++
   done

   if $_set_ipv4 ; then

      for _ipv4_address in "${ipv4_addresses_arr[@]}" ; do
         if grep -e "IN\s*A\s*$_ipv4_address" $zone_file > /dev/null 2>&1 ; then

            ## - setze neue ttl für ipv4 address
            ## -
            echononl "\t   Set TTL to the zonfile default for IPv4 address $_ipv4_address .."

            if grep -e "\s*[0-9][0-9]\s*IN\s*A\s*$_ipv4_address" $zone_file > /dev/null 2>&1 ; then
               perl -i -n -p -e "s/^(.+\s+)[0-9]{2,}(\s+IN\s+A\s+$_ipv4_address)/\1\2/" $zone_file
               if [ "$?" = "0" ]; then
                  echo_ok
                  _replaced=true
               else
                  echo_failed
                  error "Setting TTL to the zonfile default for $_ipv4_address in zone file \"$zone_file\" failed!"
               fi
            else
               echo_skipped
            fi

         fi
      done
   fi

   if $_set_ipv6 ; then

      for _ipv6_address in "${ipv6_addresses_arr[@]}" ; do
         if grep -e "IN\s*AAAA\s*$_ipv6_address" $zone_file > /dev/null 2>&1 ; then

            ## - setze neue ttl für ipv6 address
            ## -
            echononl "\t   Set TTL to the zonfile default for IPv6 address $_ipv6_address .."
            
            if grep -e "\s*[0-9][0-9]\s*IN\s*AAAA\s*$_ipv6_address" $zone_file > /dev/null 2>&1 ; then
               perl -i -n -p -e "s/^(.+\s+)[0-9]{2,}(\s+IN\s+AAAA\s+$_ipv6_address)/\1\2/" $zone_file
               if [ "$?" = "0" ]; then
                  echo_ok
                  _replaced=true
               else
                  echo_failed
                  error "Setting TTL to the zonfile default for $_ipv6_address in zone file \"$zone_file\" failed!"
               fi
            else
               echo_skipped
            fi
         fi
      done
   fi

   ## - setze neue serial
   ## -
   echo ""
   echononl "\t   Increase Serial for zone file \"`basename $zone_file`\""
   if $_replaced ; then
      perl -i -n -p -e "s#^(\s*)\s$__serial(.*)#\1 $_serial_new\2#" $zone_file > /dev/null 2>&1
      if [ "$?" = "0" ]; then
         echo_ok
         _replaced=true
      else
         echo_failed
         error "Increasing Serial for zone file \"`basename $zone_file`\" failed!"
      fi
   else
      echo_skipped
   fi

done

echo
exit
